from django import forms
#import autocomplete_light.shortcuts as al
from crispy_forms.helper import FormHelper
from crispy_forms.layout import Layout
from crispy_forms.layout import Submit
from django.contrib.contenttypes.models import ContentType
from django.contrib.auth.models import User, Group
from django.db.models.fields import BLANK_CHOICE_DASH

#from relations.forms import PersonPlaceForm, PersonInstitutionForm
from metainfo.models import Collection
from .models import AnnotationProject
from vocabularies.models import TextType
from dal import autocomplete



class LinkHighlighterForm(forms.Form):

    #relation = forms.CharField(label='Relation', widget=al.TextWidget('PersonAutocomplete'))
    relation = forms.CharField(label='Relation')


class SelectAnnotationProject(forms.Form):
    project = forms.ChoiceField(label=False, required=False)
    users_show = forms.MultipleChoiceField(label=False, required=False)
    #show_all = forms.BooleanField(label='Show all?')

    def __init__(self, *args, **kwargs):
        choices_type = ContentType.objects.filter(app_label__in=['Entities', 'Relations']).values_list('pk', 'model')
        set_ann_proj = kwargs.pop('set_ann_proj', False)
        entity_types_highlighter = kwargs.pop('entity_types_highlighter', False)
        users_show = kwargs.pop('users_show_highlighter', False)
        super().__init__(*args, **kwargs)
        CHOICES = AnnotationProject.objects.all()
        choices_user = User.objects.all()
        self.fields['project'] = forms.ChoiceField(
            choices=tuple((x.pk, x.name) for x in CHOICES),
            label=False)
        self.fields['users_show'] = forms.MultipleChoiceField(
            choices=tuple((x.pk, x.username) for x in choices_user),
            label=False)
        self.fields['types'] = forms.MultipleChoiceField(required=False, choices=choices_type, label=False)
        self.helper = FormHelper()
        self.helper.form_id = 'InitAnnotationProject'
        self.helper.form_class = 'form-inline'
        self.helper.form_method = 'GET'
        self.initial['project'] = set_ann_proj
        self.initial['types'] = entity_types_highlighter
        self.initial['users_show'] = users_show
        self.helper.add_input(Submit('Update', 'Update'))


class SelectAnnotatorAgreement(forms.Form):
    choices_metrics = (('Do_alpha', 'Disagreement Alpha Coefficient'),
                       ('multi_kappa', 'Multi Kappa (Davies and Fleiss 1982)'),
                       ('alpha', 'Krippendorff 1980'),
                       ('avg_Ao', 'Average observed agreement'),
                       ('S', 'Bennett, Albert and Goldstein 1954'),
                       ('pi', 'Scott 1955; here, multi-pi.'),
                       ('kappa', 'Kappa (Cohen 1960)'),
                       ('weighted_kappa', 'Weighted Kappa (Cohen 1968)'))
    choices_format_string = (('start_end_text', 'String & Annotation'),
                             ('start_end_text_ent', 'String, Annotation & Entity type'),
                             ('start_end_text_ent_entid', 'String, Annotation, Entity type & ID'),
                             ('start_end_text_rel_ent_entid', 'String, Annotation, Entity type, ID & relation type'))
    metrics = forms.ChoiceField(label='Metrics', choices=choices_metrics, required=True)
    format_string = forms.ChoiceField(label='Format Annotation', choices=choices_format_string, required=True)
    user_group = forms.ChoiceField(label='User group', required=False)
    gold_standard = forms.ChoiceField(label='Gold standard', required=False)

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        choices_groups = Group.objects.all()
        choices_users = User.objects.all()
        self.fields['user_group'] = forms.ChoiceField(
            choices=BLANK_CHOICE_DASH+list((x.pk, x.name) for x in choices_groups),
            required=False)
        self.fields['gold_standard'] = forms.ChoiceField(
            choices=BLANK_CHOICE_DASH+list((x.pk, x.username) for x in choices_users),
            required=False)
        self.helper = FormHelper()
        self.helper.form_id = 'SelectAnnotatorAgreement'
        self.helper.form_class = 'form-inline'
        self.helper.form_method = 'GET'
        self.helper.add_input(Submit('Update', 'Update'))


class SelectAnnotatorAgreementCollection(SelectAnnotatorAgreement):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields['anno_proj'] = forms.ChoiceField(
            label='Annotation Project',
            choices=list((x.pk, x.name) for x in AnnotationProject.objects.all()),
            required=True)
        self.fields['collection'] = forms.ChoiceField(
            choices=BLANK_CHOICE_DASH + list((x.pk, x.name) for x in Collection.objects.all()),
            required=False)
        self.fields['kind_instance'] = forms.ChoiceField(label='Entity kind Collection',
            choices=BLANK_CHOICE_DASH + list((x.pk, x.name) for x in ContentType.objects.filter(app_label="entities")),
            required=False)
        self.fields['text_type'] = forms.MultipleChoiceField(
            choices=BLANK_CHOICE_DASH + list((x.pk, x.name) for x in TextType.objects.all()),
            required=False)
        self.helper.form_id = 'SelectAnnotatorAgreementCollection'
        self.helper.form_class = 'form'
        self.helper.form_method = 'POST'
        self.order_fields(('metrics', 'anno_proj', 'format_string', 'user_group',
                                'gold_standard', 'collection', 'kind_instance', 'text_type'))


class BaseEntityHighlighterForm(forms.Form):
    HL_start = forms.IntegerField(widget=forms.HiddenInput)
    HL_end = forms.IntegerField(widget=forms.HiddenInput)
    HL_text_id = forms.CharField(widget=forms.HiddenInput)
    HL_id = forms.CharField(widget=forms.HiddenInput, required=False)

    def save(self, *args, **kwargs):
        cd = self.cleaned_data
        txt = Text.objects.get(pk=cd['HL_text_id'][5:])
        if cd['HL_id']:
            a = Annotation.objects.get(pk=cd['HL_id'])
            a.user_added = self.request.user
            a.annotation_project_id = int(self.request.session.get('annotation_project', 1))
        else:
            a = Annotation(
                start=cd['HL_start'],
                end=cd['HL_end'],
                text=txt,
                user_added=self.request.user,
                annotation_project_id=int(self.request.session.get('annotation_project', 1)))
        a.save()
        return a

    def __init__(self, *args, **kwargs):
        self.entity_type = kwargs.pop('entity_type', False)
        self.request = kwargs.pop('request', False)
        self.instance = kwargs.pop('instance', False)
        siteID = kwargs.pop('siteID', False)
        super(BaseEntityHighlighterForm, self).__init__(*args, **kwargs)
        if self.instance:
            self.fields['HL_id'].initial = self.instance
        self.helper = FormHelper()
        self.helper.form_tag = False

    def get_text_id(self):
        return self.cleaned_data['HL_text_id'][5:]


class PersonHighlighterForm(BaseEntityHighlighterForm):
    person = forms.CharField(label='Person')
    person_uri = forms.CharField(required=False, widget=forms.HiddenInput())

    def save(self, *args, **kwargs):
        x = super(PersonHighlighterForm, self).save(*args, **kwargs)
        cd = self.cleaned_data
        p = Person.get_or_create_uri(cd['person_uri'])
        if not p:
            p = GenericRDFParser(cd['person_uri'], 'Person').get_or_create()
        if len(x.entity_link.all()) > 0:
            if p != x.entity_link.all()[0]:
                x.entity_link.clear()
                x.entity_link.add(p)
        else:
            x.entity_link.add(p)
        return p

    def __init__(self, *args, **kwargs):
        super(PersonHighlighterForm, self).__init__(*args, **kwargs)
        self.helper.form_class = 'PersonEntityForm'
        if self.instance:
            a = self.instance
            ent = a.entity_link.all()[0]
            self.fields['person'].initial = ent.name+', '+ent.first_name
            self.fields['person_uri'].initial = Uri.objects.filter(entity=ent)[0].uri


class PlaceHighlighterForm(BaseEntityHighlighterForm):
    place = forms.CharField(label='Place')
    place_uri = forms.CharField(required=False, widget=forms.HiddenInput())

    def save(self, *args, **kwargs):
        x = super(PlaceHighlighterForm, self).save(*args, **kwargs)
        cd = self.cleaned_data
        p = Place.get_or_create_uri(cd['place_uri'])
        if not p:
            p = GenericRDFParser(cd['place_uri'], 'Place').get_or_create()
        if len(x.entity_link.all()) > 0:
            if p != x.entity_link.all()[0]:
                x.entity_link.clear()
                x.entity_link.add(p)
        else:
            x.entity_link.add(p)
        return p

    def __init__(self, *args, **kwargs):
        super(PlaceHighlighterForm, self).__init__(*args, **kwargs)
        self.helper.form_class = 'PlaceEntityForm'
        if self.instance:
            a = self.instance
            ent = a.entity_link.all()[0]
            self.fields['place'].initial = ent.name
            self.fields['place_uri'].initial = Uri.objects.filter(entity=ent)[0].uri


class PlaceEntityHighlighterForm(forms.Form):
    place = forms.CharField(label='Place', widget=autocomplete.ModelSelect2(url='autocomplete/place'))
    place_uri = forms.CharField(required=False, widget=forms.HiddenInput())
    HL_start = forms.IntegerField(widget=forms.HiddenInput)
    HL_end = forms.IntegerField(widget=forms.HiddenInput)
    HL_text_id = forms.CharField(widget=forms.HiddenInput)

    def save(self, *args, **kwargs):
        cd = self.cleaned_data
        pl = Place.get_or_create_uri(cd['place_uri'])
        if not pl:
            pl = GenericRDFParser(cd['place_uri'], 'Place').get_or_create()
        txt = Text.objects.get(pk=cd['HL_text_id'][5:])
        a = Annotation(
            start=cd['HL_start'],
            end=cd['HL_end'],
            text=txt,
            user_added=self.request.user,
            annotation_project_id=int(self.request.session.get('annotation_project', 1)))
        a.save()
        a.entity_link.add(pl)
        return a

    def __init__(self, *args, **kwargs):
        entity_type = kwargs.pop('entity_type', False)
        self.request = kwargs.pop('request', False)
        super(PlaceEntityHighlighterForm, self).__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.helper.form_class = 'PlaceEntityForm'
        self.helper.form_tag = False


class AddRelationHighlighterBaseForm(forms.Form):
    HL_start = forms.IntegerField(widget=forms.HiddenInput)
    HL_end = forms.IntegerField(widget=forms.HiddenInput)
    HL_text_id = forms.CharField(widget=forms.HiddenInput)
    RL_type = forms.CharField(widget=forms.HiddenInput)
    RL_pk = forms.IntegerField(widget=forms.HiddenInput)

    def save(self):
        cd = self.cleaned_data
        #x = super(AddRelationHighlighterBaseForm, self).save()
        txt = Text.objects.get(pk=cd['HL_text_id'][5:])
        ent = ContentType.objects.get(
            pk=cd['RL_type']).model_class().objects.get(pk=cd['RL_pk'])
        self.ann = Annotation(
            start=cd['HL_start'],
            end=cd['HL_end'],
            text=txt,
            user_added=self.request.user)
        self.ann.save()
        self.ann.entity_link.add(ent)
        return self.ann


class AddRelationHighlighterPersonForm(AddRelationHighlighterBaseForm):
    # relation = forms.CharField(
    #     label='Relation',
    #     widget=al.TextWidget('AddRelationPersonHighlighterAutocomplete'),
    #     )

    def save(self, *args, **kwargs):
        cd = self.cleaned_data
        x = super().save()

    def __init__(self, *args, **kwargs):
        self.request = kwargs.pop('request', False)
        self.entity_type = kwargs.pop('entity_type', False)
        super(AddRelationHighlighterPersonForm, self).__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.helper.form_tag = False

    def get_text_id(self):
        return self.cleaned_data['HL_text_id'][5:]



import spacy


class AlSpacyNER(object):
    """class that manages a spacy based NER active learning project.
    """

    def get_next_case(self):
        """get the next prediction case for the annotator
        """

    def update_model(self):

    def save(self, output_dir):
        if output_dir is not None:
            output_dir = Path(output_dir)
        if not output_dir.exists():
            output_dir.mkdir()
        nlp.to_disk(output_dir)
        print("Saved model to", output_dir)

        # test the saved model
        print("Loading from", output_dir)
        nlp2 = spacy.load(output_dir)
        for text, _ in TRAIN_DATA:
            doc = nlp2(text)
            print('Entities', [(ent.text, ent.label_) for ent in doc.ents])
            print('Tokens', [(t.text, t.ent_type_, t.ent_iob) for t in doc])


    def __init__(self, model_name, sampling, entity_type, *args, **kwargs):
        if model is not None:
            nlp = spacy.load(model)  # load existing spaCy model
            print("Loaded model '%s'" % model)
        else:
            nlp = spacy.blank('en')  # create blank Language class
            print("Created blank 'en' model")

        # create the built-in pipeline components and add them to the pipeline
        # nlp.create_pipe works for built-ins that are registered with spaCy
        if 'ner' not in nlp.pipe_names:
            ner = nlp.create_pipe('ner')
            nlp.add_pipe(ner, last=True)
        # otherwise, get it so we can add labels
        else:
            ner = nlp.get_pipe('ner')

        # add labels
        for _, annotations in TRAIN_DATA:
            for ent in annotations.get('entities'):
                ner.add_label(ent[2])

        # get names of other pipes to disable them during training
        other_pipes = [pipe for pipe in nlp.pipe_names if pipe != 'ner']
        with nlp.disable_pipes(*other_pipes):  # only train NER
            optimizer = nlp.begin_training()
            for itn in range(n_iter):
                random.shuffle(TRAIN_DATA)
                losses = {}
                for text, annotations in TRAIN_DATA:
                    nlp.update(
                        [text],  # batch of texts
                        [annotations],  # batch of annotations
                        drop=0.5,  # dropout - make it harder to memorise data
                        sgd=optimizer,  # callable to update weights
                        losses=losses)



from django.shortcuts import render, get_object_or_404
from rest_framework import viewsets, filters, mixins
from rest_framework.response import Response
from rest_framework.renderers import JSONRenderer
from rest_framework.settings import api_settings
from rest_framework.views import APIView
from django.contrib.contenttypes.models import ContentType
import pickle
from django.conf import settings

from .models import Project, TextHigh, MenuEntry, VocabularyAPI, Annotation
from .serializer import (
    projectSerializer, texthighSerializer,
    menuentrySerializer, highlightText, vocabapiSerializer, annotationSerializer,
    highlightTextTEI
)
from metainfo.models import Text
from metainfo.api_renderers import TEIBaseRenderer
from helper_functions.inter_annotator_agreement import InternalDataAgreement

if 'deep learning' in getattr(settings, "APIS_COMPONENTS", []):
    from helper_functions.dl_models import test_model




class HighlighterProjectViewSet(viewsets.ReadOnlyModelViewSet):
    """
    Endpoint that returns the menus for the different texts
    """
    #allowed_methods = ['GET']
    serializer_class = projectSerializer
    queryset = Project.objects.all()

    # def get_queryset(self):
    #     pk = self.request.query_params.get('q_pk', None)
    #     if pk:
    #         return self.queryset.filter(pk=pk)
    #     else:
    #         return self.queryset


class HighlighterTextHighViewSet(viewsets.ModelViewSet):

    serializer_class = texthighSerializer
    queryset = TextHigh.objects.all()


class HighlighterMenuEntryViewSet(viewsets.ModelViewSet):

    allowed_methods = ['GET']
    serializer_class = menuentrySerializer
    queryset = MenuEntry.objects.all()


class HighlighterAnnotationViewSet(viewsets.ModelViewSet):

    serializer_class = annotationSerializer
    queryset = Annotation.objects.all()


class HighlighterVocabularyAPIViewSet(viewsets.ReadOnlyModelViewSet):

    serializer_class = vocabapiSerializer
    queryset = VocabularyAPI.objects.all()


class HighlighterHighlightTextViewSet(
    mixins.ListModelMixin,
    mixins.RetrieveModelMixin,
    viewsets.GenericViewSet
):

    renderer_classes = (JSONRenderer, TEIBaseRenderer) + tuple(api_settings.DEFAULT_RENDERER_CLASSES)

    def get_queryset(self):
        queryset = Text.objects.all()
        pk = self.request.query_params.get('q_pk', None)
        self.user_pk = self.request.query_params.get('user', None)
        self.ann_proj_pk = self.request.query_params.get('ann_proj', None)
        self.types = self.request.query_params.get('types', None)
        self.users_show = self.request.query_params.get('users_show', None)
        if self.types:
            self.request.session['entity_types_highlighter'] = self.types.split(',')
        else:
            self.request.session['entity_types_highlighter'] = []
        if self.users_show:
            self.request.session['users_show_highlighter'] = self.users_show.split(',')
            self.users_show = self.users_show.split(',')
        else:
            self.request.session['users_show_highlighter'] = []
        self.request.session['annotation_project'] = self.ann_proj_pk
        if pk:
            return queryset.filter(pk__in=pk.split(','))
        else:
            return queryset

    def get_serializer_class(self):
        if self.request.accepted_renderer.media_type == 'application/json':
            return highlightText
        elif self.request.accepted_renderer.media_type == 'application/xml+tei':
            return highlightTextTEI
        else:
            return highlightText

    def get_serializer_context(self):
        return {'user_pk': self.user_pk,
                'ann_proj_pk': self.ann_proj_pk,
                'types': self.types,
                'users_show': self.users_show}


class AnnotatorAgreementView(APIView):

    def get(self, request, format=None):
        metrics = request.query_params.get('metrics', 'Do_alpha')
        user_group = request.query_params.get('user_group', None)
        gold_standard = request.query_params.get('gold_standard', None)
        kind_instance = request.query_params.get('kind_instance')
        instance = request.query_params.get('instance', None)
        collection = request.query_params.get('collection', None)
        ann_proj_pk = self.request.session['annotation_project']
        if instance:
            entity = ContentType.objects.get(app_label='entities',
                                             model=kind_instance).get_object_for_this_type(pk=instance)
            data = InternalDataAgreement(entity, ann_proj_pk,
                                         metrics=metrics, user_group=user_group, gold_standard=gold_standard)
        elif collection:
            entity = ContentType.objects.get(app_label='entities',
                                             model=kind_instance).model_class()
            txts = Text.objects.filter(tempentityclass__in=entity.filter(collection_id=collection))
            data = InternalDataAgreement(txts, ann_proj_pk,
                                            metrics=metrics, user_group=user_group, gold_standard=gold_standard)

        return Response({'data': data.get_html_table()})


class ShowOverlappingHighlights(APIView):

    def get(self, request, format=None):
        text_id = request.query_params.get('text_id')
        char_start = request.query_params.get('char_start')
        char_end = request.query_params.get('char_end')
        left_offset = int(request.query_params.get('left_offset', 5))
        right_offset = int(request.query_params.get('right_offset', 5))
        res = '<ul class="annotations_list">'
        txt = Text.objects.get(pk=text_id)
        for ann in Annotation.objects.filter(start__gte=char_start, end__lte=char_end, text_id=text_id):
            left_list = txt.text[:ann.start].split()
            if len(left_list) < left_offset:
                left_offset = len(left_list) - 1
            left_offset_text = ' '.join(left_list[-left_offset:])
            right_list = txt.text[ann.end:].split()
            if len(right_list) < right_offset:
                right_offset = len(right_list) - 1
            right_offset_text = ' '.join(right_list[:right_offset])
            res += '<li>{} {}{}</mark> {}</li>'.format(
                left_offset_text, ann.get_html_markup(), txt.text[ann.start:ann.end], right_offset_text)
        res += '</ul>'
        return Response({'data': res})


class TestDLModel(APIView):

    def get(self, request, format=None):
        model = request.query_params.get('model')
        text = request.query_params.get('text')

        res = test_model(model, text)
        print(res)
        return Response({'data': res})


class ActiveLearningView(APIView):

    def get(self, request, format=None):
        al_project = request.query_params.get('al_project', None)



from django.views.generic import View
from django.shortcuts import render
from django.utils.decorators import method_decorator
from django.contrib.auth.decorators import login_required
from django.contrib.contenttypes.models import ContentType
from django.conf import settings

from apis_highlighter.forms import SelectAnnotatorAgreementCollection
if 'annotator agreement' in getattr(settings, "APIS_COMPONENTS", []):
    from helper_functions.inter_annotator_agreement import InternalDataAgreement
from metainfo.models import Text, Collection, TempEntityClass


@method_decorator(login_required, name='dispatch')
class ComputeAgreement(View):

    def get(self, request):
        form = SelectAnnotatorAgreementCollection()
        return render(request, 'calculate_agreement.html', {'form': form})

    def post(self, request, *args, **kwargs):
        form = SelectAnnotatorAgreementCollection(request.POST)
        if form.is_valid():
            gold_standard = form.cleaned_data.get('gold_standard')
            user_group = form.cleaned_data.get('user_group')
            metrics = form.cleaned_data.get('metrics')
            format_string = form.cleaned_data.get('format_string')
            ann_proj_pk = form.cleaned_data.get('anno_proj')
            collection = form.cleaned_data.get('collection')
            kind_instance = form.cleaned_data.get('kind_instance')
            text_type = form.cleaned_data.get('text_type')
            met_dict = dict(form.fields['metrics'].choices)
            title1 = met_dict[form.cleaned_data['metrics']]
            txt = Text.objects.all()
            ent = False
            if kind_instance:
                col = Collection.objects.get(pk=collection)
                ent = ContentType.objects.get_for_id(kind_instance).model_class()
                ent = ent.objects.filter(collection=col)
            elif collection:
                col = Collection.objects.get(pk=collection)
                ent = TempEntityClass.objects.filter(collection=col)
            if ent:
                txt = txt.filter(tempentityclass__in=ent).distinct()
            if text_type:
                txt = txt.filter(kind_id__in=text_type)
            tab1, tab2 = InternalDataAgreement(
                txt, ann_proj_pk, user_group,
                gold_standard=gold_standard, metrics=metrics,
                combine=True, format_string=format_string).get_html_table()
            return render(request, 'calculate_agreement.html', {
                'form': form, 'tab1': tab1, 'tab2': tab2, 'title1': title1})



from django.contrib import admin

from .models import *

admin.site.register(AnnotationProject)
admin.site.register(Annotation)
admin.site.register(Project)
admin.site.register(TextHigh)
admin.site.register(VocabularyAPI)
admin.site.register(MenuEntry)



from rest_framework import serializers
from .models import Annotation, Project, TextHigh, MenuEntry, VocabularyAPI
from django.http import HttpResponse
from rest_framework.renderers import JSONRenderer
from rest_framework.parsers import JSONParser
from helper_functions.highlighter import highlight_text, highlight_textTEI

from vocabularies.serializers import UserAccSerializer


class JSONResponse(HttpResponse):
    """
    An HttpResponse that renders its content into JSON.
    """
    def __init__(self, data, **kwargs):
        content = JSONRenderer().render(data)
        kwargs['content_type'] = 'application/json'
        super(JSONResponse, self).__init__(content, **kwargs)


class annotationSerializer(serializers.HyperlinkedModelSerializer):
    """Serializer for the Annotation model.
    """

    class Meta:
        model = Annotation
        fields = ('start', 'end', 'uri')


class texthighSerializer(serializers.HyperlinkedModelSerializer):
    #project = serializers.HyperlinkedRelatedField(many=False, view_name='api:project-detail', read_only=True)

    class Meta:
        model = TextHigh
        fields = ('text_id', 'text_class')


class vocabapiSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        model = VocabularyAPI
        fields = ('name', 'api_endpoint', 'method')


class menuentrySerializer(serializers.HyperlinkedModelSerializer):
    parent = serializers.PrimaryKeyRelatedField(many=False, read_only=True)
    api = vocabapiSerializer(many=False, read_only=True)

    class Meta:
        model = MenuEntry
        fields = ('id', 'name', 'kind', 'api', 'parent')


class projectSerializer(serializers.HyperlinkedModelSerializer):
    texthigh_set = texthighSerializer(many=True, read_only=True)
    menuentry_set = menuentrySerializer(many=True, read_only=True)

    class Meta:
        model = Project
        fields = ('name', 'base_url', 'store_text', 'texthigh_set', 'menuentry_set')


class annotationSerializer(serializers.HyperlinkedModelSerializer):
    parent = serializers.PrimaryKeyRelatedField(many=False, read_only=True)
    user_added = UserAccSerializer(many=False, read_only=True)

    class Meta:
        model = Annotation
        fields = ('start', 'end', 'text', 'user_added', 'parent')


class highlightText(serializers.BaseSerializer):
    func_offset = highlight_text

    def to_representation(self, obj):
        html_return = self.func_offset(
            obj,
            user=self.context['user_pk'],
            ann_proj=self.context['ann_proj_pk'],
            types=self.context['types'],
            users_show=self.context['users_show'])
        res = {
            'text_id': obj.pk,
            'text': html_return
        }
        return res

    def __init__(self, *args, **kwargs):
        self.user = kwargs.pop('user', False)
        self.ann_proj = kwargs.pop('ann_proj', False)
        super().__init__(*args, **kwargs)

class highlightTextTEI(highlightText):
    func_offset = highlight_textTEI



from django.db import models
from django.contrib.auth.models import User
from gm2m import GM2MField
import reversion
from django.contrib.contenttypes.models import ContentType
import re
from django.conf import settings


##############################################
#
#   Project classes
#
##############################################


class Project(models.Model):
    """Holds information on registered project.
    Id is used by the JavaScript function to target the right endpoints
    """
    name = models.CharField(max_length=255)  # name of the project registered
    user = models.ForeignKey(User, on_delete=models.CASCADE)  # foreignkey to the User who created the project
    description = models.TextField(blank=True, null=True)
    base_url = models.URLField(blank=True, null=True)    # optional base URL to restrict highlights to a base URL
    store_text = models.BooleanField(default=False) # Whether to store the text in the tool or work with ids only.

    def __str__(self):
        return self.name

    class Meta:
        db_table = 'highlighter_project'


class TextHigh(models.Model):
    """ Holds unstructured text associated with
    one ore many entities/relations.
    """
    title = models.CharField(max_length=255, blank=True, null=True)
    text_choices = (('ft', 'Full Text'), ('id', 'Text ID'), ('cl', 'Text Class'))
    text_type = models.CharField(max_length=3, choices=text_choices, default='cl')
    uri = models.URLField(blank=True, null=True) # uri of vocab used to identify the text type
    text = models.TextField(blank=True, null=True)
    text_id = models.PositiveIntegerField(blank=True, null=True) # UID to identify the text within the project. Allows to not upload the texts.
    text_class = models.CharField(max_length=255, blank=True, null=True)
    project = models.ForeignKey(Project, blank=True, null=True,
                                on_delete=models.SET_NULL)

    def __str__(self):
        return self.title

    class Meta:
        db_table = 'highlighter_texthigh'


class AnnotationProject(models.Model):
    '''Every Project can have several Annotation Projects. Annotation Projects are used to track the precission etc.
    of automatic annotation tools.
    '''
    name = models.CharField(max_length=255)
    description = models.TextField(blank=True, null=True)

    def __str__(self):
        return self.name

    class Meta:
        db_table = 'highlighter_annotationproject'


@reversion.register()
class Annotation(models.Model):
    """Class storing highlights in full-texts
    """
    set_highlighter = getattr(
        settings,
        'APIS_HIGHLIGHTER_ENTITIES',
        ('entities.Person',
            'entities.Institution',
            'entities.Place',
            'entities.Event',
            'entities.Work',
            'relations.PersonPerson',
            'relations.PersonPlace',
            'relations.PersonInstitution',
            'relations.PersonEvent',
            'relations.PersonWork',
            'relations.InstitutionPlace',
            'relations.InstitutionEvent',
            'relations.InstitutionWork',
            'relations.InstitutionInstitution',
            'relations.PlaceEvent',
            'relations.PlaceWork',
            'relations.PlacePlace',
            'relations.EventWork',
            'relations.EventEvent',
            'relations.WorkWork'))
    status_choices = (('del', 'deleted'), ('ap', 'approved'))
    start = models.PositiveIntegerField()   # number of string to start highlight
    end = models.PositiveIntegerField()     # number of string to end highlight
    entity_link = GM2MField(*set_highlighter)  # generic field to store the relation object
    entity_candidate = models.ManyToManyField('metainfo.UriCandidate', blank=True)
    orig_string = models.CharField(max_length=255, blank=True, null=True)    # string originally highlighted
    text = models.ForeignKey('metainfo.Text', on_delete=models.CASCADE)
    parent = models.ForeignKey('self', related_name='parent_annotation',
                               blank=True, null=True,
                               on_delete=models.SET_NULL)    #parent annotations are used to allow for a stacked design of the annotation project
    user_added = models.ForeignKey(User, blank=True, null=True,
                                   on_delete=models.SET_NULL) #changed from default=12
    annotation_project = models.ForeignKey(AnnotationProject,
                                           blank=True, null=True,
                                           on_delete=models.CASCADE)
    status = models.CharField(max_length=4, choices=status_choices, blank=True, null=True)

    def __str__(self):
        return "{}".format(self.pk)

    def trim_whitespaces(self, char):
        test = True
        while test and char < self.text_length:
            if self.text.text[char] == ' ':
                test = True
                char += 1
            else:
                return char

    def annotation_hash(self, format_string='start_end_text_ent_entid'):
        """
        Function that returns a hash of the annotaion used to calculate inter-annotator agreement.

        :return:
        """
        matching = {'start': 'start', 'end': 'end', 'text': 'text_id'}
        f_list = format_string.split('_')
        if 'start' not in f_list or 'end' not in f_list or 'text' not in f_list:
            raise AttributeError('Start, end and text must be present in the format_string parameter')
        res = ''
        self.text_length = len(self.text.text)
        self.start = self.trim_whitespaces(self.start)
        self.end = self.trim_whitespaces(self.end)
        for f in f_list:
            if f in matching.keys():
                res += str(getattr(self, matching[f]))
            else:
                ent_link = self.entity_link.all()
                if len(ent_link) > 0:
                    cont_lst = str(ContentType.objects.get_for_model(ent_link[0])).split()
                    if f == 'ent':
                        res += ''.join(cont_lst)
                    elif f == 'entid':
                        for cont in cont_lst:
                            if hasattr(ent_link[0], 'related_'+cont.strip()):
                                res += str(getattr(ent_link[0], 'related_'+cont.strip()+'_id'))
                            elif hasattr(ent_link[0], 'related_'+cont.strip()+'A'):
                                res += str(getattr(ent_link[0], 'related_'+cont.strip()+'A_id'))
                            elif hasattr(ent_link[0], 'related_'+cont.strip()+'B'):
                                res += str(getattr(ent_link[0], 'related_'+cont.strip()+'B_id'))
                            res += '-'
                        res = res[:-1]
                    elif f == 'rel':
                        res += str(getattr(ent_link[0], 'relation_type_id'))
                else:
                    res += 'NONE'
            res += '_'
        return res[:-1]

    def get_html_markup(self):
        if not len(self.entity_link.all()) == 1:
            return None
        rel_entity = self.entity_link.all()[0]
        if hasattr(rel_entity, 'relation_type'):
            entity_kind = str(rel_entity.relation_type.pk)
        elif hasattr(rel_entity, 'kind'):
            entity_kind = str(rel_entity.kind.pk)
        entity_type = type(rel_entity).__name__
        entity_type_app = ContentType.objects.get(model=entity_type).app_label
        ent_lst_pk = []
        user_added = self.user_added.username[:2]
        if entity_type_app.lower() == 'relations':
            for x in dir(rel_entity):
                c = re.match('related_\w+_id', x)
                if c:
                    ent_lst_pk.append(str(getattr(rel_entity, c.group(0))))
        start_span = '''<mark class="highlight hl_text_{}" data-hl-type="simple" data-hl-start="{}"
            data-hl-end="{}" data-hl-text-id="{}" data-hl-ann-id="{}" data-entity-class="{}" data-entity-pk="{}"
            data-related-entity-pk="{}" data-entity-type="{}" data-user-added={}>'''.format(entity_kind, self.start, self.end,
                                                                        self.text_id, self.pk, entity_type, rel_entity.pk,
                                                                                 ','.join(ent_lst_pk),
                                                                                 entity_type_app, user_added)
        return start_span

    class Meta:
        db_table = 'highlighter_annotation'

##############################################
#
#   Configuration Classes
#
##############################################


class VocabularyAPI(models.Model):
    """Class storing the information of the endpoints of the vocabularies.
    """
    method_lst = (('l','local'), ('o', 'open skos'))
    name = models.CharField(max_length=255, null=True, blank=True)
    api_endpoint = models.CharField(max_length=255, null=True, blank=True)
    method = models.CharField(max_length=2, choices=method_lst, default='l')

    def __str__(self):
        return self.name

    class Meta:
        db_table = 'highlighter_vocabularyapi'


class MenuEntry(models.Model):
    """Class that defines the menu structure of the context menu
    """
    choices_kind = (('txt', 'Text field'), ('frm', 'Form'),
                    ('m', 'menu entry'), ('fn', 'Javascript function'))
    kind = models.CharField(max_length=4, choices=choices_kind, default='txt')
    name = models.CharField(max_length=255, null=True, blank=True)    # Charfield used when txt is as type of entry specified
    api = models.ForeignKey(VocabularyAPI, blank=True, null=True,
                            on_delete=models.SET_NULL)  # Foreignkey used to link the API entry in case api is set as kind
    parent = models.ForeignKey('self', related_name='parent_menuEntry',
                               blank=True, null=True, on_delete=models.SET_NULL)
    project = models.ForeignKey(Project, null=True, blank=True,
                                on_delete=models.CASCADE)

    def __str__(self):
        return self.name

    class Meta:
        db_table = 'highlighter_menuentry'


#######################################################################
#
# DL-Classes
#
#######################################################################


class ActiveLearningProject(models.Model):
    """Used to store active learning project settings.
    """
    choices = ((''))
    name = models.CharField(max_length=255)
    sampling_strategy = models.CharField(max_length=4, choices=choices)
    rebuild = models.IntegerField()
    collection = models
    log_dir = models.CharField(max_length=255)

    class Meta:
        db_table = 'highlighter_activelearningproject'



#!/usr/bin/python
# -*- coding: utf-8 -*-
import autocomplete_light.shortcuts as al
from django.utils.encoding import force_text
from vocabularies.models import (PersonInstitutionRelation, PersonPersonRelation, PersonPlaceRelation,
                                 PersonEventRelation, InstitutionEventRelation, PlaceEventRelation,
                                 RelationBaseClass, VocabsBaseClass, InstitutionInstitutionRelation,
                                 InstitutionPlaceRelation, PersonWorkRelation, InstitutionWorkRelation,
                                 PlaceWorkRelation, EventWorkRelation, PlacePlaceRelation)

import requests
import re, copy

lst_schemes = {'Oebl_brg': 'http://acdh.oeaw.ac.at/vocabs/professions/oebl/'}


class OpenSkosAutocompleteBase(al.AutocompleteListTemplate):
    autocomplete_template = 'autocomplete/openskos.html'
    base_url = 'https://clarin.oeaw.ac.at/vocabs/api/find-concepts'

    attrs = {
        'data-autocomplete-minimum-characters': 3,
        'placeholder': u'Start typing to get suggestions',
        'class': 'autocomplete-ort-uri form-control'
    }

    def parse_search(self):
        q = self.request.GET.get('q')
        if q:
            qp = re.match(r'^(?P<scheme>\[[^\]]+\])?(?P<query>.*)$', q)
            if qp.group('scheme'):
                s = qp.group('scheme').strip('[]').split('#')
                self.scheme = [x.strip() for x in s]
            if qp.group('query'):
                s = qp.group('query').split('#')
                self.query = [x.strip() for x in s]
        else:
            self.scheme = []
            self.query = []

    def choices_for_request(self):
        def run_through_dict(dict2, items, level=0, list2=[]):
            for k, v in dict2.items():
                f = dict()
                if 'broader' not in items[k].keys():
                    level = 0
                #print(items[k])
                ff = level+(len(items[k]['prefLabel'][0]))
                if level > 0:
                    f['name'] = '|{:_>{}s}'.format(items[k]['prefLabel'][0], ff)
                else:
                    f['name'] = items[k]['prefLabel'][0]
                f['uri'] = k
                list2.append(f)
                if isinstance(v, dict):
                    level += 1
                    list2 = run_through_dict(v, items, level, list2)
                    level -= 1
            return list2

        def set_path(someDict, path, value):
            for x in path[::-1]:
                value = {x: value}
            return deepupdate(someDict, value)


        def deepupdate(original, update):
            for key, value in original.items():
                if not key in update:
                    update[key] = value
                elif isinstance(value, dict):
                    deepupdate(value, update[key])
            return update

        def get_elem_uri(dict2, value):
            if value not in dict2.keys():
                headers = {'Content-Type': 'application/json'}
                data = {'format': 'json', 'id': value}
                r = requests.get(self.base_url, params=data, headers=headers)
                res = r.json()
                dict2[value] = res
            return dict2, dict2[value]


        self.parse_search()
        headers = {'Content-Type': 'application/json'}
        scheme_str = ' OR '.join(self.scheme)
        prev_elem = None
        counter = 0
        choices = []
        for x in self.query:
            if len(x) > 0:
                counter += 1
                if prev_elem:
                    data = {'format': 'json',
                            'q': '(inScheme:"{}") AND \
                            {}* AND ({})'.format(scheme_str, x, prev_elem)}
                else:
                    data = {'format': 'json',
                            'q': '(inScheme:"{}") AND {}*'.format(scheme_str, x)}
                r = requests.get(self.base_url, params=data, headers=headers)
                res = r.json()
                if counter < len(self.query):
                    prev_elem2 = []
                    for x in res['response']['docs']:
                        prev_elem2.append(x['uri'])
                    prev_elem = ' OR '.join(['broader:"{}"'.format(x) for x in prev_elem2])
                else:
                    dict_res = {i['uri']: i for i in res['response']['docs']}
                    lst_proof = []
                    dict_res2 = dict()
                    for x in dict_res.keys():
                        if 'broader' not in dict_res[x].keys():
                            lst_proof.append(x)
                            dict_res2[x] = dict()
                    dict_res_bak = copy.deepcopy(dict_res)
                    lst_proof = []
                    lst_obj_fin = []
                    for x in dict_res_bak.keys():
                        if dict_res_bak[x]['uri'] not in lst_proof:
                            list_obj = []
                            check = dict_res_bak[x]['uri']
                            while check:
                                list_obj.insert(0, check)
                                if 'broader' in dict_res[check].keys():
                                    dict_res = get_elem_uri(dict_res, dict_res[check]['broader'][0])[0]
                                    check = dict_res[check]['broader'][0]
                                else:
                                    dict_res2[check] = dict()
                                    check = False
                        lst_obj_fin.append(list_obj)
                    lst_obj_fin = sorted(lst_obj_fin, key=len)
                    for a in lst_obj_fin:
                        dict_res2 = set_path(dict_res2, a, dict())
                    choices = run_through_dict(dict_res2, dict_res)
        return choices

    def __init__(self, *args, **kwargs):
        super(OpenSkosAutocompleteBase, self).__init__(*args, **kwargs)


class VocabsAutocompleteBase(al.AutocompleteListTemplate):
    autocomplete_template = 'autocomplete/vocabs_base.html'

    attrs = {
        'data-autocomplete-minimum-characters': 3,
        'placeholder': u'Start typing to get suggestions',
        #'class': 'autocomplete-ort-uri form-control'
    }

    def choices_for_request(self):
        q = self.request.GET.get('q')
        res = []
        for d in self.vocabs:
            for x in d.objects.filter(name__icontains=q):
                res.append(x)
        return res


class VocabsAutocompleteBase2(al.AutocompleteModelTemplate):
    autocomplete_template = 'autocomplete/vocabs_base2.html'
    attrs = {
    'display': 'block'
    }

    def choice_label(self, choice):
        # if type(choice) is VocabsBaseClass:
        #     choice = RelationBaseClass.objects.get(pk=choice.pk)
        # d = choice
        # res = getattr(choice, self.choice_attr)
        # while d.parent_class:
        #     f = d.parent_class
        #     if type(f) is VocabsBaseClass:
        #         f = RelationBaseClass.objects.get(pk=f.pk)
        #     res = getattr(f, self.choice_attr) + ' >> ' + res
        #     d = f
        if self.reverse:
            res = choice.label_reverse
        else:
            res = choice.label
        return force_text(res)

    def choices_for_request(self):
        if self.request.user.is_superuser:
            self.choices = self.choices.all()
        else:
            self.choices = self.choices.filter(userAdded__groups__in=self.request.user.groups.all())
        return super(VocabsAutocompleteBase2, self).choices_for_request()

    def order_choices(self, choices):
        lst_choices = []
        for x in choices:
            lst2 = [x]
            x.label = self.choice_label(x)
            d = x
            while d.parent_class:
                lst2.append(d.parent_class)
                d = d.parent_class
            lst_choices.append(lst2)
        result = []
        lst_choices_sort = sorted(lst_choices, key=lambda x: '_'.join([z.name for z in reversed(x)]))
        for x in lst_choices_sort:
            result.append(x[0])
        return result

    def __init__(self, *args, **kwargs):
        if self.reverse:
            self.choice_attr = 'name_reverse'
        else:
            self.choice_attr = 'name'
        super(VocabsAutocompleteBase2, self).__init__(*args, **kwargs)



class OSProfessionsAutocomplete(OpenSkosAutocompleteBase):
    scheme = ['http://acdh.oeaw.ac.at/vocabs/professions/oebl/']


# class VCPersonInstitutionAutocomplete(VocabsAutocompleteBase):
#     vocabs = [PersonInstitutionRelation]

class VCPersonInstitutionAutocomplete(VocabsAutocompleteBase2):
    search_fields = ('name',)
    choices = PersonInstitutionRelation.objects.all()
    reverse = False


class VCPersonInstitutionReverseAutocomplete(VCPersonInstitutionAutocomplete):
    search_fields = ('name_reverse', )
    reverse = True


class VCPersonPlaceAutocomplete(VocabsAutocompleteBase2):
    search_fields = ('name',)
    choices = PersonPlaceRelation.objects.all()
    reverse = False


class VCPersonPlaceReverseAutocomplete(VCPersonPlaceAutocomplete):
    search_fields = ('name_reverse', )
    reverse = True


class VCPersonPersonAutocomplete(VocabsAutocompleteBase2):
    search_fields = ('name', )
    choices = PersonPersonRelation.objects.all()
    reverse = False


class VCPersonEventAutocomplete(VocabsAutocompleteBase2):
    search_fields = ('name', )
    choices = PersonEventRelation.objects.all()
    reverse = False


class VCPersonEventReverseAutocomplete(VCPersonEventAutocomplete):
    search_fields = ('name_reverse', )
    reverse = True


class VCInstitutionEventAutocomplete(VocabsAutocompleteBase2):
    search_fields = ('name', )
    choices = InstitutionEventRelation.objects.all()
    reverse = False


class VCInstitutionEventReverseAutocomplete(VCInstitutionEventAutocomplete):
    search_fields = ('name_reverse', )
    reverse = True


class VCInstitutionInstitutionAutocomplete(VocabsAutocompleteBase2):
    search_fields = ('name', )
    choices = InstitutionInstitutionRelation.objects.all()
    reverse = False


class VCInstitutionInstitutionReverseAutocomplete(VCInstitutionInstitutionAutocomplete):
    search_fields = ('name_reverse',)
    reverse = True


class VCInstitutionPlaceAutocomplete(VocabsAutocompleteBase2):
    search_fields = ('name',)
    choices = InstitutionPlaceRelation.objects.all()
    reverse = False


class VCInstitutionPlaceReverseAutocomplete(VCInstitutionPlaceAutocomplete):
    search_fields = ('name_reverse',)
    reverse = True


class VCPlaceEventAutocomplete(VocabsAutocompleteBase2):
    search_fields = ('name', )
    choices = PlaceEventRelation.objects.all()
    reverse = False


class VCPlaceEventReverseAutocomplete(VCPlaceEventAutocomplete):
    search_fields = ('name_reverse', )
    reverse = True


class VCPlacePlaceAutocomplete(VocabsAutocompleteBase2):
    search_fields = ('name', )
    choices = PlacePlaceRelation.objects.all()
    reverse = False


class VCPlacePlaceReverseAutocomplete(VCPlacePlaceAutocomplete):
    search_fields = ('name_reverse',)
    reverse = True


class VCPersonPersonReverseAutocomplete(VCPersonPersonAutocomplete):
    search_fields = ('name_reverse',)
    reverse = True


class VCPersonWorkAutocomplete(VocabsAutocompleteBase2):
    search_fields = ('name', )
    choices = PersonWorkRelation.objects.all()
    reverse = False


class VCPersonWorkReverseAutocomplete(VCPersonWorkAutocomplete):
    search_fields = ('name_reverse',)
    reverse = True


class VCInstitutionWorkAutocomplete(VocabsAutocompleteBase2):
    search_fields = ('name', )
    choices = InstitutionWorkRelation.objects.all()
    reverse = False


class VCInstitutionWorkReverseAutocomplete(VCInstitutionWorkAutocomplete):
    search_fields = ('name_reverse',)
    reverse = True


class VCPlaceWorkAutocomplete(VocabsAutocompleteBase2):
    search_fields = ('name', )
    choices = PlaceWorkRelation.objects.all()
    reverse = False


class VCPlaceWorkReverseAutocomplete(VCPlaceWorkAutocomplete):
    search_fields = ('name_reverse',)
    reverse = True


class VCEventWorkAutocomplete(VocabsAutocompleteBase2):
    search_fields = ('name', )
    choices = EventWorkRelation.objects.all()
    reverse = False


class VCEventWorkReverseAutocomplete(VCEventWorkAutocomplete):
    search_fields = ('name_reverse',)
    reverse = True


al.register(OSProfessionsAutocomplete)
al.register(VCPersonInstitutionAutocomplete)
al.register(VCPersonPlaceAutocomplete)
al.register(VCPersonPersonAutocomplete)
al.register(VCPersonEventAutocomplete)
al.register(VCInstitutionEventAutocomplete)
al.register(VCPlaceEventAutocomplete)
al.register(VCPersonPersonReverseAutocomplete)
al.register(VCInstitutionInstitutionAutocomplete)
al.register(VCInstitutionInstitutionReverseAutocomplete)
al.register(VCPersonInstitutionReverseAutocomplete)
al.register(VCPersonPlaceReverseAutocomplete)
al.register(VCPersonEventReverseAutocomplete)
al.register(VCInstitutionEventReverseAutocomplete)
al.register(VCPlaceEventReverseAutocomplete)
al.register(VCInstitutionPlaceAutocomplete)
al.register(VCInstitutionPlaceReverseAutocomplete)
al.register(VCPersonWorkAutocomplete)
al.register(VCPersonWorkReverseAutocomplete)
al.register(VCInstitutionWorkAutocomplete)
al.register(VCInstitutionWorkReverseAutocomplete)
al.register(VCPlaceWorkAutocomplete)
al.register(VCPlaceWorkReverseAutocomplete)
al.register(VCEventWorkAutocomplete)
al.register(VCEventWorkReverseAutocomplete)
al.register(VCPlacePlaceAutocomplete)
al.register(VCPlacePlaceReverseAutocomplete)



from django.conf.urls import url
from django.conf import settings

urlpatterns = []

if 'annotator agreement' in getattr(settings, "APIS_COMPONENTS", []):
    from apis_highlighter.views import ComputeAgreement
    urlpatterns.append(url(r'^agreement/$', ComputeAgreement.as_view(), name='agreement'))



import os
from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

bower_json = {
"dependencies": {
    "jquery": "^3.2.1",
    "bootstrap": "^3.3.7",
    "bootstrap-multiselect": "^0.9.13",
    "bootstrap-datepicker": "^1.7.0",
    "jquery-tablesort": "^0.0.11",
    "leaflet": "^1.1.0",
    "tooltipster": "^4.2.5",
    "leaflet.markercluster": "Leaflet.markercluster#^1.0.6",
  },
  "resolutions": {
    "jquery": "1.9.1 - 3"
  }
}

setup(
    name='apis-highlighter',
    version='0.8.3',
    packages=find_packages(),
    include_package_data=True,
    license='MIT License',  # example license
    description='Addon to APIS webapplication that allows to annotate texts',
    long_description=README,
    url='https://www.apis.acdh.oeaw.ac.at/',
    author='Matthias Schlögl',
    author_email='matthias.schloegl@oeaw.ac.at',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.11',  # replace "X.Y" as appropriate
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',  # example license
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    install_requires=[
        'calmjs.bower',
        'Django>=2.0',
        'django-autocomplete-light>=3.2.10',
        'django-crispy-forms>=1.7.0',
        'django-filter>=1.0.4',
        'django-tables2>=1.12.0',
        'djangorestframework>=3.6.2',
        'djangorestframework-csv>=2.0.0',
        'djangorestframework-xml>=1.3.0',
    ],
    bower_json=bower_json,
)
